/**
 * Error thrown when the item returned from a canonical definition file is not the expected item.
 * @extends Error
 */
import {Canonical, CanonicalDefinition} from "./Canonical";
import {Instantiable, isInstantiable} from "@extollo/di";

export class InvalidCanonicalExportError extends Error {
    constructor(name: string) {
        super(`Unable to import canonical item from "${name}". The default export of this file is invalid.`)
    }
}

export class CanonicalInstantiable<T> extends Canonical<Instantiable<T>> {
    public async initCanonicalItem(definition: CanonicalDefinition): Promise<Instantiable<T>> {
        if ( isInstantiable(definition.imported.default) ) {
            return this.app().make(definition.imported.default)
        }

        if ( isInstantiable(definition.imported[definition.canonicalName.split(':').reverse()[0]]) ) {
            return this.app().make(definition.imported[definition.canonicalName.split(':').reverse()[0]])
        }

        throw new InvalidCanonicalExportError(definition.originalName)
    }
}